/* Pinout:
 *
 *
 * */
#include <stdio.h>
#include <string.h>
#include <Keypad.h>

#define A_POT0 A0
#define A_POT1 A1
#define A_POT2 A2
/* Rotary not supported yet */
#define A_LIN0 A4
#define A_LIN1 A7
#define A_LIN2 A6
#define A_TOGGLES A5

byte kpins[8] = {0xFF, 2, 3, 4, 5, 6, 7, 8};
byte but_pins[] = {10, 11, 12, 13};

/* Keypad */
#define ROWS 4
#define COLS 3
char keys[ROWS][COLS] = {
  {'1','2','3'},
  {'4','5','6'},
  {'7','8','9'},
  {'*','0','#'}
};
byte rowPins[ROWS] = {
  kpins[2],
  kpins[7],
  kpins[6],
  kpins[4]};
byte colPins[COLS] = {
  kpins[3],
  kpins[1],
  kpins[5]};
Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS );

void setup() {
  pinMode(A0, INPUT);
  pinMode(A1, INPUT);
  pinMode(A2, INPUT);
  pinMode(A3, INPUT);
  pinMode(A4, INPUT);
  pinMode(A5, INPUT);
  pinMode(A6, INPUT);
  pinMode(A7, INPUT);

  /* Configure input pins */
  for(int i=0; i<(sizeof(but_pins)/sizeof(byte)); i++)
  {
    pinMode(but_pins[i], INPUT_PULLUP);
  }

  Serial.begin(9600);
}

void readPots(int* val) {
  val[0] = analogRead(A_POT0);
  val[1] = analogRead(A_POT1);
  val[2] = analogRead(A_POT2);
}

void readSliders(int* val) {
  val[0] = analogRead(A_LIN0);
  val[1] = analogRead(A_LIN1);
  val[2] = analogRead(A_LIN2);
}

uint16_t readSelector() {
  /* Decode analog value into bits */
  /* Selector is not wired yet */
  return 0;
}

uint8_t readButtons(void) {
  uint8_t bmask = 0;

  for(int i=0; i<(sizeof(but_pins)/sizeof(byte)); i++)
  {
    bmask |= (digitalRead(but_pins[i]) == 0) << i;
  }

  return bmask;
}

uint8_t readToggles(void) {
  int val = analogRead(A_TOGGLES);
  /* compensate for analog noise on b0 */
  val += 20;
  /* decode to binary value */
  val *= 8;
  val /= 1024;

  val = ~val & 0x7;

  return (uint8_t)val;
}

uint16_t readKeypad(void) {
  /* TODO: latch value until key is released */
  return (uint16_t)keypad.getKey();
}

char dst[100] = {0};

struct __attribute__((packed)) inputs {
    uint8_t header[4];
    uint16_t len;
    uint8_t  crc;
    uint16_t pot[3];
    uint16_t slider[3];
    uint16_t selector;
    uint16_t keypad;
    uint8_t  buttons;
    uint8_t  toggles;
} inputs;

void loop() {
  memcpy(inputs.header, "SENS", 4);
  inputs.len = 18;

  readPots(inputs.pot);
  readSliders(inputs.slider);
  inputs.selector = readSelector();
  inputs.keypad = readKeypad();
  inputs.buttons = readButtons();
  inputs.toggles = readToggles();

  memcpy(dst, &inputs, sizeof(inputs));

  Serial.write((char*)&dst, sizeof(inputs));

  delay(50);
}
